SystemOrganization addCategory: #'Pier-LinkChecker'!

!HTTPSocket class methodsFor: '*pier-linkchecker' stamp: 'lr 5/28/2009 14:37'!
httpHead: aString
	| serverName serverAddr port sock bare page index connectToHost connectToPort |

	Socket initializeNetwork.
	bare := (aString asLowercase beginsWith: 'http://') 
		ifTrue: [ aString copyFrom: 8 to: aString size ]
		ifFalse: [ aString ].
	bare := bare copyUpTo: $#.
	serverName := bare copyUpTo: $/.
	page := bare copyFrom: serverName size + 1 to: bare size.
	(serverName includes: $:) 
		ifTrue: [
			index := serverName indexOf: $:.
			port := (serverName copyFrom: index + 1 to: serverName size) asNumber.
			serverName := serverName copyFrom: 1 to: index - 1 ]
		ifFalse: [ port := self defaultPort ].
	page size = 0 ifTrue: [ page := '/' ].

	(self shouldUseProxy: serverName)
		ifFalse: [ 
			connectToHost := serverName.
			connectToPort := port ]
		ifTrue:  [
			page := 'http://' , serverName , ':' , port printString , page.
			connectToHost := self httpProxyServer.
			connectToPort := self httpProxyPort ].

	serverAddr := NetNameResolver addressForName: connectToHost timeout: 20.
	serverAddr ifNil: [ ^ nil ].

	sock := HTTPSocket new.
	^ [	sock connectTo: serverAddr port: connectToPort.
		(sock waitForConnectionFor: 30)
			ifFalse: [ ^ nil ].
		sock sendCommand: 'HEAD ' , page , ' HTTP/1.0' , String crlf , 
			'Host: ' , serverName , ':' , port printString , String crlf.
		sock getResponseUpTo:  String crlf , String crlf ignoring: String cr ]
			ensure: [ sock destroy ]! !

PRObject subclass: #PULinkChange
	instanceVariableNames: 'enabled source target status'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-LinkChecker'!

!PULinkChange class methodsFor: 'instance creation' stamp: 'lr 5/28/2009 11:46'!
on: aLink
	^ self basicNew initializeOn: aLink! !

!PULinkChange methodsFor: 'private' stamp: 'lr 5/28/2009 20:39'!
httpHead: aString
	"Answer an array with response code and effective URL, or nil."

	| url socket |
	url := Url absoluteFromText: aString.
	(url isKindOf: HttpUrl)
		ifFalse: [ ^ nil ].
	[ socket := HTTPSocket
		initHTTPSocket: url
		wait: (HTTPSocket deadlineSecs: 1)
		ifError: [ ^ Array with: 404 with: aString ] ]
			on: NameLookupFailure
			do: [ :err | ^ Array with: 404 with: aString ].
	^ [ [ socket sendCommand: ('HEAD ' , ((HTTPSocket shouldUseProxy: url authority)
		ifTrue: [ url printString ] ifFalse: [ url fullPath ]) ,  ' HTTP/1.0' , String crlf) ,
			('Host: ' , url authority , String crlf) , 
			(HTTPSocket userAgentString , String crlf) ,
			(HTTPSocket classPool at: #HTTPProxyCredentials).
		socket header: (socket getResponseUpTo: String crlf , String crlf) first.
		Array with: socket responseCode asInteger with: (socket getHeader: 'location' default: aString) ]
			on: Error do: [ :err | nil ] ]
			ensure: [ socket destroy ]! !

!PULinkChange methodsFor: 'initialization' stamp: 'jf 9/24/2009 17:05'!
initializeOn: aLink
	self initialize.
	source := aLink.
	target := aLink copy! !

!PULinkChange methodsFor: 'testing' stamp: 'lr 5/31/2009 20:15'!
isEnabled
	^ self source ~= self target! !

!PULinkChange methodsFor: 'testing' stamp: 'lr 5/28/2009 18:21'!
isValid
	^ self status first beginsWith: 'Valid'! !

!PULinkChange methodsFor: 'accessing' stamp: 'lr 5/28/2009 13:29'!
owner
	^ source owner! !

!PULinkChange methodsFor: 'action' stamp: 'lr 5/28/2009 20:49'!
refresh
	status := nil! !

!PULinkChange methodsFor: 'action' stamp: 'lr 5/28/2009 20:38'!
repair
	self target isInternal
		ifTrue: [ self repairInternal ].
	self target isExternal
		ifTrue: [ self repairExternal ]! !

!PULinkChange methodsFor: 'action' stamp: 'lr 5/28/2009 20:49'!
repairExternal
	self status size < 3 ifTrue: [ ^ self ].
	self target reference: self status third second.
	self refresh! !

!PULinkChange methodsFor: 'action' stamp: 'lr 5/28/2009 20:49'!
repairInternal
	| reference matcher |
	reference := self target reference.
	matcher := [ :structure |
		| newTarget |
		newTarget := structure lookupPath: reference.
		newTarget isNil
			ifFalse: [ self refresh. ^ self target target: newTarget; update ] ].
	self owner parents reversed do: matcher.
	self owner enumerator all do: matcher.
	self owner root enumerator everything do: matcher.
	(reference := self target reference copyAfterLast: $/) isEmpty 
		ifTrue: [ ^ self ].
	self owner enumerator everything do: matcher.
	self owner root enumerator everything do: matcher! !

!PULinkChange methodsFor: 'accessing' stamp: 'lr 5/28/2009 13:29'!
source
	^ source! !

!PULinkChange methodsFor: 'accessing' stamp: 'lr 5/28/2009 20:47'!
sourceString
	^ self source reference! !

!PULinkChange methodsFor: 'accessing' stamp: 'lr 5/28/2009 18:08'!
status
	"Answer an array with status message and status string."
	
	^ status ifNil: [ status := self updateStatus ]! !

!PULinkChange methodsFor: 'accessing' stamp: 'lr 5/28/2009 11:52'!
target
	^ target! !

!PULinkChange methodsFor: 'accessing' stamp: 'lr 5/28/2009 20:47'!
targetString
	^ self target reference! !

!PULinkChange methodsFor: 'accessing' stamp: 'lr 5/28/2009 20:49'!
targetString: aString
	self targetString = aString 
		ifTrue: [ ^ self ].
	self target reference: aString.
	self refresh! !

!PULinkChange methodsFor: 'private' stamp: 'jf 11/3/2009 11:12'!
updateStatus
	| response |
	target isExternal ifTrue: [
		response := self httpHead: target url.
		(response isNil)
			ifTrue: [ ^ Array with: 'Unknown' with: 'white' ].
		(response first between: 300 and: 399)
			ifTrue: [ ^ Array with: 'Redirect ' , response first greaseString with: 'yellow' with: response ].
		(response first between: 200 and: 299)
			ifTrue: [ ^ Array with: 'Valid ' , response first greaseString with: 'green' ].
		 ^ Array with: 'Error ' , response first greaseString with: 'red' ].
	^ target isBroken
		ifTrue: [ Array with: 'Error' with: 'error' ]
		ifFalse: [ Array with: 'Valid' with: 'green' ]! !

WAComponent subclass: #PULinkChangesView
	instanceVariableNames: 'changes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-LinkChecker'!

!PULinkChangesView class methodsFor: 'instance-creation' stamp: 'lr 5/28/2009 13:21'!
on: aCollection
	^ self new initializeOn: aCollection! !

!PULinkChangesView methodsFor: 'actions' stamp: 'lr 5/31/2009 20:20'!
apply: aChange
	self inform: 'Not implemented, yet.'! !

!PULinkChangesView methodsFor: 'accessing' stamp: 'lr 5/28/2009 13:24'!
changes
	^ changes! !

!PULinkChangesView methodsFor: 'initialization' stamp: 'jf 9/24/2009 17:06'!
initializeOn: aCollection
	self initialize.
	changes := aCollection! !

!PULinkChangesView methodsFor: 'rendering' stamp: 'lr 5/31/2009 20:19'!
renderButtonsOn: html
	html submitButton
		with: 'Refresh'.
	html submitButton
		callback: [ self answer ];
		with: 'Close'! !

!PULinkChangesView methodsFor: 'rendering' stamp: 'lr 5/28/2009 13:25'!
renderContentOn: html
	html form: [
		self renderTableOn: html.
		self renderButtonsOn: html ]! !

!PULinkChangesView methodsFor: 'rendering' stamp: 'lr 5/31/2009 20:18'!
renderTableHeadOn: html
	html tableHeading: 'Owner'.
	html tableHeading: 'Old'.
	html tableHeading: 'New'.
	html tableHeading: 'Status'.
	html tableHeading! !

!PULinkChangesView methodsFor: 'rendering' stamp: 'lr 5/28/2009 13:41'!
renderTableOn: html
	changes isEmpty
		ifTrue: [ ^ html paragraph: 'No broken links detected.' ].
	html table: [
		html tableRow: [ self renderTableHeadOn: html ].
		changes keysAndValuesDo: [ :key :value |
			html tableRow
				class: 'even' if: key even;
				with: [ self renderTableRow: value on: html ] ] ]! !

!PULinkChangesView methodsFor: 'rendering' stamp: 'lr 5/31/2009 20:20'!
renderTableRow: aChange on: html
	html tableData: [
		html anchor
			goto: (self context structure: aChange owner);
			with: aChange owner absolutePath ].
	html tableData: aChange sourceString.
	html tableData: [
		html textInput
			value: aChange targetString;
			style: 'background-color: ' , aChange status second;
			callback: [ :value | aChange targetString: value ] ].
	html tableData: aChange status first.
	html tableData: [ 
		html submitButton
			disabled: aChange isEnabled not;
			callback: [ self apply: aChange ];
			with: 'Apply' ]! !

!GRPackage class methodsFor: '*pier-linkchecker' stamp: 'jf 10/28/2009 22:04'!
pierLinkChecker
	^ self new
		name: 'Pier-LinkChecker';
		addDependency: 'Pier-Model';
		addDependency: 'Pier-Security';
		addDependency: 'Pier-Seaside';
		yourself! !

PUReportWidget subclass: #PULinksWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-LinkChecker'!

!PULinksWidget class methodsFor: 'testing' stamp: 'lr 4/6/2009 13:00'!
isAbstract
	^ false! !

!PULinksWidget class methodsFor: 'accessing' stamp: 'lr 5/28/2009 11:33'!
label
	^ 'Links'! !

!PULinksWidget methodsFor: 'private' stamp: 'lr 4/22/2009 17:16'!
buildReport
	| component |
	component := super buildReport.
	component addColumn: (MAColumn new
		renderer: [ :value :html | self renderLink: value on: html ];
		formatter: [ :value | value reference ];
		accessor: #yourself;
		label: 'Link';
		yourself).
	component addColumn: (MAColumn new
		accessor: #(class label);
		label: 'Kind';
		filter: true;
		yourself).
	component addColumn: (MAColumn new
		renderer: [ :value :html | self renderStructure: value owner on: html ];
		formatter: [ :value | value absolutePath ];
		accessor: #owner;
		label: 'Owner';
		filter: true;
		yourself).
	component addColumn: (MAColumn new
		accessor: #isEmbedded;
		label: 'Embedded';
		filter: true;
		yourself).
	component addColumn: (MAColumn new
		accessor: #isBroken;
		label: 'Broken';
		filter: true;
		yourself).
	^ component! !

!PULinksWidget methodsFor: 'actions' stamp: 'lr 9/4/2009 17:33'!
check
	"Checks the currently displayed links."
	
	| changes change |
	changes := OrderedCollection new.
	self report cache do: [ :link |
		change := PULinkChange on: link.
		change isValid
			ifFalse: [ changes add: change repair ] ].
	self 
		show: (PULinkChangesView on: changes)
		onAnswer: [ :answer | self refresh ]! !

!PULinksWidget methodsFor: 'rendering' stamp: 'lr 5/28/2009 11:39'!
renderButtonsOn: html
	super renderButtonsOn: html.
	html submitButton
		callback: [ self check ];
		with: 'Check'! !

!PULinksWidget methodsFor: 'rendering' stamp: 'lr 4/22/2009 16:59'!
renderLink: aLink on: html
	| document |
	(aLink isKindOf: PRValueLink)
		ifTrue: [ document := PRText content: aLink text ]
		ifFalse: [
			document := aLink copy.
			document setChildren: #(); parameters: #(); embedded: false.
			document add: (PRText content: aLink reference).
			document propertyAt: 'title' put: aLink text ].
	html rendererInstance
		continue: document
		in: self report on: html! !

!PULinksWidget methodsFor: 'accessing' stamp: 'lr 4/22/2009 17:02'!
rows
	| links |
	links := OrderedCollection new: 1024.
	(self context enumeratorFor: self context root) everything do: [ :structure |
		structure outgoingReferences
			continue: [ :link | false ];
			do: [ :each | 
				each owner isNil 
					ifFalse: [ links addLast: each ] ] ].
	^ links! !
